$PS = "/usr/bin/ps";
$GREP = "/bin/grep";
$CAT = "/bin/cat";

#
# This parse routine takes an instance name and returns the
# Server Root path of that particular instance. The file is
# processed one line at a time.
#
# INPUT:
#
# $_[0] == A Variable String
# $_[1] == File Name
# $_[2] == Instance Name (The intance name, normally the logical host name)
#
# OUTPUT:
#
# The corresponding value. 
#
SERVERROOT="<msg.RootPath>";
sub parse {
	my $String = shift;
	my $FileName = shift;
	my $Instance = shift;
	my $Msg = "msg.Instances";
	my $var, $value, $rvalue;

	return ($SERVERROOT);
}
#
# INPUT:
# 
# $_[0] == Process String.
# $_[1] == RootPath (To distinguish between multiple instances.)
#
# OUTPUT:
# 
# process ID list. 
#
sub getproc {
  	my $ProcStr = shift;
	my $RootPath = shift;
	my $pid, $psargs;

 	open (PSOUT, "$PS -ea -o pid,args | $GREP $ProcStr | $GREP -v grep | $GREP $RootPath |");
        while (<PSOUT>) {
               next if not /$ProcStr/;
               chomp;
               ($pid, $psargs) = split;
        }
        close (PSOUT);
        return ($pid);
}

#
# connection() probes for the service status by openning a socket connection,
# checking for the server response and then sending a protocol logout command.
#
# The <sock> call is set to a configurable alarm interval. If the socket stalls, 
# the process return with a false.
#
# Succeed: 110
# Fail:  100
# 
# INPUT:
#
# $_[0] == Server Name
# $_[1] == Port Number
# $_[2] == Server Response
# $_[3] == Client Logout Command
#
# OUTPUT:
#
# exit_code
#

sub connection {
	use Socket;
	my $ret = 110;
	my $ServerName = $_[0];
	my $PORT = $_[1];
	my $Timeout = 160;  # secs < 180
	my $SRP = parse("RootPath", "", "");

	$iaddr = inet_aton($ServerName);
	$paddr = sockaddr_in($PORT, $iaddr);
	$proto = getprotobyname('tcp');

	socket(SOCK, AF_INET, SOCK_STREAM, $proto) 	or return (100);
	connect(SOCK, $paddr)				or return (100);

	my $CMD = $_[$i];
	send SOCK, "$CMD \n", 50;
	close(SOCK);

	return $ret;
}

# Function to get the ports of all the messaging Services
#
# INPUT:
#
# $_[0] == Server Root
# $_[1] == Config variable.
#
# OUTPUT:
#
# The port of the requested messaging Service
#

sub get_port {
	my $ServerRootPath = shift;
	my $configVar = shift;
	my $GotPort;

	if ( -f "${ServerRootPath}/sbin/configutil") {
        	$GotPort = `$ServerRootPath/sbin/configutil -o $configVar`;
		if ($GotPort eq "" ){
			$GotPort = 49994;
		}
	} else {
		if ($configVar eq "local.watcher.port") { $GotPort = 49994; }
	}
	return ($GotPort);
}

#
# get_admin_server_root()
#	Gets the admin Server Root of the admin server which this 
#	messaging Server is using
# Possible return values: 
# 	Returns the Admin_Server_Root or logs an error in case the Admin Server
# 	Root is not found
# INPUT:
# $_[0] == Server Root
#
sub get_admin_server_root {
	my $ServerRootPath = shift;
	my $InputFile="$ServerRootPath/lib/config-templates/DevsetupDefaults.properties";
	my $admin_file;
	# Open the DevSetupDefaults.properties file and get the AdminServerRoot
	unless (open(INFD,"<$InputFile")) {
        	print "get_admin_server_root Error: Unable to open $InputFile.\n";
		return ($ServerRootPath);
	}
	@InputText = <INFD>;
	close (INFD);
	foreach $line (@InputText){
		if($line=~ /(ADMINSERVER_SERVERROOT_CONF)\s=\s(.*)/){
			$admin_file="$2";
		}
	}
	unless (open(OUTFD,"<$admin_file")) {
		print "get_admin_server_root Error: Unable to open $admin_file.\n";
		return ($ServerRootPath);
	}
	@OutputText = <OUTFD>;
	close (OUTFD);
	return ( $OutputText[0] );
}

#
# check_stored_state() 
# 	checks to see if stored is in an init state or not.
#
# Possible stored process state and the corresponding return code:
#
#       0: ready state  1: init state 2: zombie state. 
#

sub check_stored_state {
	my $ServerRootPath = shift;
	my $MsgInstance = shift;
  	my $ProcStr = "stored";

	my $PidFile = "$ServerRootPath/config/pidfile.store";
	my $pid, $state, $line;

	#print "ProcStr = $ProcStr\n";
	#print "ServerRootPath = $ServerRootPath\n";

	my $StoredPid = getproc($ProcStr, "$ServerRootPath");

	#print "in check_stored_state: PID = $StoredPid and PidFile = $PidFile\n";

	if ( -f $PidFile ) { 
 		open (CATOUT, "$CAT $PidFile |");
        	while (<CATOUT>) {
                	chomp;	     # remove newline.
			$line = $_;  # read one line at a time.
			if ( $line =~ /[0-9]/ ) {
				$pid = $line;
			} else {
				$state = $line;  
			}
		next;
        	}
        	close (CATOUT);
	}
	#print "StoredPid = $StoredPid, pid = $pid and state = $state \n"; 

	if ($StoredPid =~ /[0-9]/ &&  "$StoredPid" eq "$pid") {
		if ($state eq "init") { 
			#print "init state\n";		
			return (1);
		} else {
			#print "ready state\n";
			return (0);
		}
	} 
	#print "zombie state\n";
	return (2);  
}
